function flux = uniqueLP(model)
%
%uniqueLP
%
% finds a unique solution to the problem
%   max f'x 
%   S x = b subject to
%   lb <= x <= ub 
%
% as described in
%   K Smallbone, E Simeonidis (2009). Flux balance analysis: 
%   A geometric perspective. J Theor Biol 258: 311-315
%   http://dx.doi.org/10.1016/j.jtbi.2009.01.027
%
% Usage: flux = uniqueLP(model)
%
% model (the following fields are required - others can be supplied)
%   S            Stoichiometric matrix
%   b            Right hand side = 0
%   lb           Lower bounds
%   ub           Upper bounds
%
%kieran, 20 april 2010

% set algorithm parameter values
lpTol           = 1e-6;
lpEps           = 0*lpTol;
showProgress    = true;

A = model.S;
b = model.b;
L = model.lb;
U = model.ub;

% ensure column vectors
b = b(:); L = L(:); U = U(:);

% Remove negligible elements
J = any(A,2); A = A(J,:); b = b(J);

% presolve
v = nan(size(L));
J = (U-L < lpTol);
v(J) = (L(J)+U(J))/2;
J = find(isnan(v));

if showProgress
    fprintf('%s\n\n','the algorithm can take some time to run, be patient');
    fprintf('%s\t%g\n\n%s\t@%s\n','# reactions:',length(v),'iteration #0',datestr(now,16)); 
end

L0 = L; U0 = U;
for k = J(:)'
    f = zeros(length(v),1); f(k) = -1;
    [dummy,opt,conv] = easyLP(f,A,b,L0,U0);
    if conv, vL = max(-opt,L(k));	else vL = L(k);	end
    [dummy,opt,conv] = easyLP(-f,A,b,L0,U0);
    if conv, vU = min(opt,U(k));    else vU = U(k);	end
    if abs(vL) < lpTol, vL = 0; end
    if abs(vU) < lpTol, vU = 0; end
    vM = (vL + vU)/2; if abs(vM) < lpTol, vM = 0; end
    if abs(vU - vL) < lpTol
        vL = (1-sign(vM)*lpEps)*vM; 
        vU = (1+sign(vM)*lpEps)*vM; 
    end
    L(k) = vL; U(k) = vU;
end

v = nan(size(L)); 
J = (U-L < lpTol);
v(J) = (L(J)+U(J))/2; v = v.*(abs(v)>lpTol);

if showProgress
    fprintf('%s\t\t%g\n%s\t\t%g\n\n','fixed:',sum(J),'@ zero:',sum(v==0));
end

% iterate
J  = find(U-L >= lpTol); 
n   = 1;
mu  = [];
Z   = [];

while ~isempty(J)
    
    if showProgress
        fprintf('%s #%g\t@%s\n','iteration',n,datestr(now,16)); 
    end
    
    if n == 1
        M = zeros(size(L));
    else
        M = (L+U)/2;
    end
    
    mu(:,n) = M;                                                %#ok<AGROW>    
    allL = L; allU = U; allA = A; allB = b;    
    [a1,a2] = size(A);
    
    % build new matrices
    for k = 1:(n-1)
        [b1,b2] = size(allA);
        f = sparse(b2+2*a2,1); f((b2+1):end) = -1;
        opt = -Z(k);
        allA = [allA,sparse(b1,2*a2);
            speye(a2,a2),sparse(a2,b2-a2),-speye(a2),speye(a2);
            f(:)'];                                             %#ok<AGROW>
        allB = [allB;mu(:,k);opt];                              %#ok<AGROW>
        allL = [allL;zeros(2*a2,1)];                            %#ok<AGROW>
        allU = [allU;inf*ones(2*a2,1)];                         %#ok<AGROW>
    end
    
    [b1,b2] = size(allA);
    f = sparse(b2+2*a2,1); f((b2+1):end) = -1;
    allA = [allA,sparse(b1,2*a2);
        speye(a2,a2),sparse(a2,b2-a2),-speye(a2),speye(a2)];	%#ok<AGROW>
    allB = [allB;M];                                            %#ok<AGROW>
    allL = [allL;zeros(2*a2,1)];                                %#ok<AGROW>
    allU = [allU;inf*ones(2*a2,1)];                             %#ok<AGROW>
    
    [v,opt,conv] = easyLP(f,allA,allB,allL,allU);
    if ~conv, disp('error: no convergence'); flux = (L+U)/2; return; end    
    
    opt = ceil(-opt/eps)*eps;
    Z(n) = opt;                                                 %#ok<AGROW>    
    allA = [allA; sparse(f(:)')];                               %#ok<AGROW>
    allB = [allB; -opt];                                        %#ok<AGROW>
    
    for k = J(:)'        
        f = zeros(length(allL),1); f(k) = -1;
        [dummy,opt,conv] = easyLP(f,allA,allB,allL,allU);
        if conv, vL = max(-opt,L(k));	else vL = L(k); end
        [dummy,opt,conv] = easyLP(-f,allA,allB,allL,allU);
        if conv, vU = min(opt,U(k));    else vU = U(k);	end
        if abs(vL) < lpTol, vL = 0; end
        if abs(vU) < lpTol, vU = 0; end
        vM = (vL + vU)/2; if abs(vM) < lpTol, vM = 0; end
        if abs(vU - vL) < lpTol, vL = (1-lpEps)*vM; vU = (1+lpEps)*vM; end
        L(k) = vL; U(k) = vU;
    end
    
    v = nan(size(L)); 
    J = (U-L < lpTol);
    v(J) = (L(J)+U(J))/2; v = v.*(abs(v)>lpTol);

    if showProgress
        fprintf('%s\t\t%g\n%s\t\t%g\n\n','fixed:',sum(J),'@ zero:',sum(v==0));
    end
    
    n = n+1;
    J = find(U-L >= lpTol);   
    
    flux = v;
end
